]> git.cworth.org Git - empires-server/blobdiff - lmno.js
tictactoe: Use a "game-state" event instead of a series of "move" events
[empires-server] / lmno.js
diff --git a/lmno.js b/lmno.js
index abcdba2ef220bd6aff8375bb3190d0cfbc87c727..9998c734a2dc63b8f0f0943a928a1d76c73e4102 100644 (file)
--- a/lmno.js
+++ b/lmno.js
@@ -4,6 +4,7 @@ const body_parser = require("body-parser");
 const session = require("express-session");
 const bcrypt = require("bcrypt");
 const path = require("path");
+const nunjucks = require("nunjucks");
 
 try {
   var lmno_config = require("./lmno-config.json");
@@ -40,32 +41,38 @@ app.use(session({
   saveUninitialized: false
 }));
 
+nunjucks.configure("templates", {
+  autoescape: true,
+  express: app
+});
+
 /* Load each of our game mini-apps. */
-var empires = require("./empires");
+const engines = {
+  empires: require("./empires"),
+  tictactoe: require("./tictactoe")
+};
 
 class LMNO {
   constructor() {
-    this.ids = {};
+    this.games = {};
   }
 
   generate_id() {
-    return [null,null,null,null].map(() => LMNO.letters.charAt(Math.floor(Math.random() * LMNO.letters.length))).join('');
+    return Array(4).fill(null).map(() => LMNO.letters.charAt(Math.floor(Math.random() * LMNO.letters.length))).join('');
   }
 
-  create_game(engine) {
+  create_game(engine_name) {
     do {
       var id = this.generate_id();
-    } while (id in this.ids);
+    } while (id in this.games);
+
+    const engine = engines[engine_name];
 
-    const game = new empires.Game();
+    const game = new engine.Game(id);
 
-    this.ids[id] = {
-       id: id,
-       engine: engine,
-       game: game
-    };
+    this.games[id] = game;
 
-    return id;
+    return game;
   }
 }
 
@@ -101,8 +108,8 @@ function lmno_canonize(id) {
 
 app.post('/new/:game_engine', (request, response) =>  {
   const game_engine = request.params.game_engine;
-  const game_id = lmno.create_game(game_engine);
-  response.send(JSON.stringify(game_id));
+  const game = lmno.create_game(game_engine);
+  response.send(JSON.stringify(game.id));
 });
 
 /* Redirect any requests to a game ID at the top-level.
@@ -121,36 +128,42 @@ app.get('/[a-zA-Z0-9]{4}', (request, response) => {
     return;
   }
 
-  const game = lmno.ids[game_id];
+  const game = lmno.games[game_id];
   if (game === undefined) {
       response.sendStatus(404);
       return;
   }
-  response.redirect(301, `/${game.engine}/${game.id}/`);
+  response.redirect(301, `/${game.meta.identifier}/${game.id}/`);
 });
 
 /* LMNO middleware to lookup the game. */
-app.use('/empires/:game_id([a-zA-Z0-9]{4})', (request, response, next) => {
+app.use('/:engine([^/]+)/:game_id([a-zA-Z0-9]{4})', (request, response, next) => {
+  const engine = request.params.engine;
   const game_id = request.params.game_id;
   const canon_id = lmno_canonize(game_id);
 
-  /* Redirect user to page with the canonical ID in it. */
-  if (game_id !== canon_id) {
-    const new_url = request.originalUrl.replace("/empires/" + game_id,
-                                                "/empires/" + canon_id);
+  /* Redirect user to page with the canonical ID in it, also ensuring
+   * that the game ID is _always_ followed by a slash. */
+  const has_slash = new RegExp(`^/${engine}/${game_id}/`);
+  if (game_id !== canon_id ||
+      ! has_slash.test(request.originalUrl))
+  {
+    const old_path = new RegExp(`/${engine}/${game_id}/?`);
+    const new_path = `/${engine}/${canon_id}/`;
+    const new_url = request.originalUrl.replace(old_path, new_path);
     response.redirect(301, new_url);
     return;
   }
 
   /* See if there is any game with this ID. */
-  const game = lmno.ids[game_id];
+  const game = lmno.games[game_id];
   if (game === undefined) {
     response.sendStatus(404);
     return;
   }
 
   /* Stash the game onto the request to be used by the game-specific code. */
-  request.game = game.game;
+  request.game = game;
   next();
 });
 
@@ -174,6 +187,7 @@ function auth_admin(request, response, next) {
 
 app.get('/logout', (request, response) => {
   request.session.user = undefined;
+  request.session.destroy();
 
   response.send("You are now logged out.");
 });
@@ -184,7 +198,7 @@ app.get('/login', (request, response) => {
     return;
   }
 
-  response.sendFile(path.join(__dirname, './login.html'));
+  response.render('login.html');
 });
 
 app.post('/login', async (request, response) => {
@@ -205,24 +219,48 @@ app.post('/login', async (request, response) => {
   return;
 });
 
-/* A stats page (only available to admin users) */
-app.get('/stats/', auth_admin, (request, response) => {
-  let active = 0;
-  let idle = 0;
+/* API to set uer profile information */
+app.put('/profile', (request, response) => {
+  const nickname = request.body.nickname;
+  if (nickname) {
+    request.session.nickname = nickname;
+    request.session.save();
+  }
+  response.send();
+});
+
+/* An admin page (only available to admin users, of course) */
+app.get('/admin/', auth_admin, (request, response) => {
+  let active = [];
+  let idle = [];
 
-  for (let id in lmno.ids) {
-    if (lmno.ids[id].game.clients.length)
-      active++;
-   else
-      idle++;
+  for (let id in lmno.games) {
+    if (lmno.games[id].clients.length)
+      active.push(lmno.games[id]);
+    else
+      idle.push(lmno.games[id]);
   }
-  response.send(`<html><body>Active games: ${active}.<br>
-Idle games: ${idle}</body></html>`);
+  response.render('admin.html', { test: "foobar", games: { active: active, idle: idle}});
 });
 
 
 /* Mount sub apps. only _after_ we have done all the middleware we need. */
-app.use('/empires/[a-zA-Z0-9]{4}/', empires.app);
+for (let key in engines) {
+  const engine = engines[key];
+
+  /* Add routes that are common to all games. */
+  engine.router.get('/', (request, response) => {
+    const game = request.game;
+
+    if (! request.session.nickname)
+      response.render('choose-nickname.html', { game_name: game.meta.name });
+    else
+      response.render(`${game.meta.identifier}-game.html`);
+  });
+
+  /* And mount the whole router at the path for the game. */
+  app.use(`/${engine.Game.meta.identifier}/[a-zA-Z0-9]{4}/`, engine.router);
+}
 
 app.listen(4000, function () {
   console.log('LMNO server listening on localhost:4000');